@@ -100,6 +100,46 @@ static void gotweb_free_repo_dir(struct repo_dir *);
 
 struct server *gotweb_get_server(uint8_t *, uint8_t *);
 
+static int
+gotweb_reply(struct request *c, int status, const char *ctype,
+    struct gotweb_url *location)
+{
+	const char	*csp;
+
+	if (status != 200 && fcgi_printf(c, "Status: %d\r\n", status) == -1)
+		return -1;
+
+	if (location) {
+		if (fcgi_puts(c->tp, "Location: ") == -1 ||
+		    gotweb_render_url(c, location) == -1 ||
+		    fcgi_puts(c->tp, "\r\n") == -1)
+			return -1;
+	}
+
+	csp = "Content-Security-Policy: default-src 'self'; "
+	    "script-src 'none'; object-src 'none';\r\n";
+	if (fcgi_puts(c->tp, csp) == -1)
+		return -1;
+
+	if (ctype && fcgi_printf(c, "Content-Type: %s\r\n", ctype) == -1)
+		return -1;
+
+	return fcgi_puts(c->tp, "\r\n");
+}
+
+static int
+gotweb_reply_file(struct request *c, const char *ctype, const char *file,
+    const char *suffix)
+{
+	int r;
+
+	r = fcgi_printf(c, "Content-Disposition: attachment; "
+	    "filename=%s%s\r\n", file, suffix ? suffix : "");
+	if (r == -1)
+		return -1;
+	return gotweb_reply(c, 200, ctype, NULL);
+}
+
 void
 gotweb_process_request(struct request *c)
 {
@@ -171,7 +211,7 @@ gotweb_process_request(struct request *c)
 	if (qs->action == BLOBRAW) {
 		const uint8_t *buf;
 		size_t len;
-		int binary;
+		int binary, r;
 
 		error = got_get_repo_commits(c, 1);
 		if (error)
@@ -182,15 +222,12 @@ gotweb_process_request(struct request *c)
 			goto render;
 
 		if (binary)
-			error = gotweb_render_content_type_file(c,
-			    "application/octet-stream", qs->file, NULL);
+			r = gotweb_reply_file(c, "application/octet-stream",
+			    qs->file, NULL);
 		else
-			error = gotweb_render_content_type(c, "text/plain");
-
-		if (error) {
-			log_warnx("%s: %s", __func__, error->msg);
+			r = gotweb_reply(c, 200, "text/plain", NULL);
+		if (r == -1)
 			goto done;
-		}
 
 		for (;;) {
 			error = got_object_blob_read_block(&len, blob);
@@ -226,22 +263,16 @@ gotweb_process_request(struct request *c)
 		if (error2)
 			goto render;
 		if (binary) {
-			fcgi_puts(c->tp, "Status: 302\r\n");
-			fcgi_puts(c->tp, "Location: ");
-			gotweb_render_url(c, &url);
-			fcgi_puts(c->tp, "\r\n\r\n");
+			gotweb_reply(c, 302, NULL, &url);
 			goto done;
 		}
 	}
 
 	if (qs->action == RSS) {
-		error = gotweb_render_content_type_file(c,
-		    "application/rss+xml;charset=utf-8",
-		    repo_dir->name, ".rss");
-		if (error) {
-			log_warnx("%s: %s", __func__, error->msg);
-			goto err;
-		}
+		const char *ctype = "application/rss+xml;charset=utf-8";
+
+		if (gotweb_reply_file(c, ctype, repo_dir->name, ".rss") == -1)
+			goto done;
 
 		error = got_get_repo_tags(c, D_MAXSLCOMMDISP);
 		if (error) {
@@ -254,11 +285,8 @@ gotweb_process_request(struct request *c)
 	}
 
 render:
-	error = gotweb_render_content_type(c, "text/html");
-	if (error) {
-		log_warnx("%s: %s", __func__, error->msg);
-		goto err;
-	}
+	if (gotweb_reply(c, 200, "text/html", NULL) == -1)
+		goto done;
 	html = 1;
 
 	if (gotweb_render_header(c->tp) == -1)
@@ -734,7 +762,6 @@ gotweb_free_repo_dir(struct repo_dir *repo_dir)
 		free(repo_dir->owner);
 		free(repo_dir->description);
 		free(repo_dir->url);
-		free(repo_dir->age);
 		free(repo_dir->path);
 	}
 	free(repo_dir);
@@ -761,29 +788,6 @@ gotweb_free_transport(struct transport *t)
 	free(t);
 }
 
-const struct got_error *
-gotweb_render_content_type(struct request *c, const char *type)
-{
-	const char *csp = "default-src 'self'; script-src 'none'; "
-		"object-src 'none';";
-
-	fcgi_printf(c,
-	    "Content-Security-Policy: %s\r\n"
-	    "Content-Type: %s\r\n\r\n",
-	    csp, type);
-	return NULL;
-}
-
-const struct got_error *
-gotweb_render_content_type_file(struct request *c, const char *type,
-    const char *file, const char *suffix)
-{
-	fcgi_printf(c, "Content-type: %s\r\n"
-	    "Content-disposition: attachment; filename=%s%s\r\n\r\n",
-	    type, file, suffix ? suffix : "");
-	return NULL;
-}
-
 void
 gotweb_get_navs(struct request *c, struct gotweb_url *prev, int *have_prev,
     struct gotweb_url *next, int *have_next)
@@ -1333,7 +1337,7 @@ done:
 	error = got_get_repo_owner(&repo_dir->owner, c);
 	if (error)
 		goto err;
-	error = got_get_repo_age(&repo_dir->age, c, NULL, TM_DIFF);
+	error = got_get_repo_age(&repo_dir->age, c, NULL);
 	if (error)
 		goto err;
 	error = gotweb_get_clone_url(&repo_dir->url, srv, repo_dir->path,
@@ -1363,7 +1367,6 @@ gotweb_init_repo_dir(struct repo_dir **repo_dir, const char *dir)
 	(*repo_dir)->owner = NULL;
 	(*repo_dir)->description = NULL;
 	(*repo_dir)->url = NULL;
-	(*repo_dir)->age = NULL;
 	(*repo_dir)->path = NULL;
 
 	return NULL;
@@ -1461,9 +1464,10 @@ done:
 	return error;
 }
 
-const struct got_error *
-gotweb_get_time_str(char **repo_age, time_t committer_time, int ref_tm)
+int
+gotweb_render_age(struct template *tp, time_t committer_time, int ref_tm)
 {
+	struct request *c = tp->tp_arg;
 	struct tm tm;
 	long long diff_time;
 	const char *years = "years ago", *months = "months ago";
@@ -1474,69 +1478,67 @@ gotweb_get_time_str(char **repo_age, time_t committer_time, int ref_tm)
 	char datebuf[64];
 	size_t r;
 
-	*repo_age = NULL;
-
 	switch (ref_tm) {
 	case TM_DIFF:
 		diff_time = time(NULL) - committer_time;
 		if (diff_time > 60 * 60 * 24 * 365 * 2) {
-			if (asprintf(repo_age, "%lld %s",
+			if (fcgi_printf(c, "%lld %s",
 			    (diff_time / 60 / 60 / 24 / 365), years) == -1)
-				return got_error_from_errno("asprintf");
+				return -1;
 		} else if (diff_time > 60 * 60 * 24 * (365 / 12) * 2) {
-			if (asprintf(repo_age, "%lld %s",
+			if (fcgi_printf(c, "%lld %s",
 			    (diff_time / 60 / 60 / 24 / (365 / 12)),
 			    months) == -1)
-				return got_error_from_errno("asprintf");
+				return -1;
 		} else if (diff_time > 60 * 60 * 24 * 7 * 2) {
-			if (asprintf(repo_age, "%lld %s",
+			if (fcgi_printf(c, "%lld %s",
 			    (diff_time / 60 / 60 / 24 / 7), weeks) == -1)
-				return got_error_from_errno("asprintf");
+				return -1;
 		} else if (diff_time > 60 * 60 * 24 * 2) {
-			if (asprintf(repo_age, "%lld %s",
+			if (fcgi_printf(c, "%lld %s",
 			    (diff_time / 60 / 60 / 24), days) == -1)
-				return got_error_from_errno("asprintf");
+				return -1;
 		} else if (diff_time > 60 * 60 * 2) {
-			if (asprintf(repo_age, "%lld %s",
+			if (fcgi_printf(c, "%lld %s",
 			    (diff_time / 60 / 60), hours) == -1)
-				return got_error_from_errno("asprintf");
+				return -1;
 		} else if (diff_time > 60 * 2) {
-			if (asprintf(repo_age, "%lld %s", (diff_time / 60),
+			if (fcgi_printf(c, "%lld %s", (diff_time / 60),
 			    minutes) == -1)
-				return got_error_from_errno("asprintf");
+				return -1;
 		} else if (diff_time > 2) {
-			if (asprintf(repo_age, "%lld %s", diff_time,
+			if (fcgi_printf(c, "%lld %s", diff_time,
 			    seconds) == -1)
-				return got_error_from_errno("asprintf");
+				return -1;
 		} else {
-			if (asprintf(repo_age, "%s", now) == -1)
-				return got_error_from_errno("asprintf");
+			if (fcgi_puts(tp, now) == -1)
+				return -1;
 		}
 		break;
 	case TM_LONG:
 		if (gmtime_r(&committer_time, &tm) == NULL)
-			return got_error_from_errno("gmtime_r");
+			return -1;
 
 		s = asctime_r(&tm, datebuf);
 		if (s == NULL)
-			return got_error_from_errno("asctime_r");
+			return -1;
 
-		if (asprintf(repo_age, "%s UTC", datebuf) == -1)
-			return got_error_from_errno("asprintf");
+		if (fcgi_puts(tp, datebuf) == -1 ||
+		    fcgi_puts(tp, " UTC") == -1)
+			return -1;
 		break;
 	case TM_RFC822:
 		if (gmtime_r(&committer_time, &tm) == NULL)
-			return got_error_from_errno("gmtime_r");
+			return -1;
 
 		r = strftime(datebuf, sizeof(datebuf),
 		    "%a, %d %b %Y %H:%M:%S GMT", &tm);
 		if (r == 0)
-			return got_error(GOT_ERR_NO_SPACE);
+			return -1;
 
-		*repo_age = strdup(datebuf);
-		if (*repo_age == NULL)
-			return got_error_from_errno("asprintf");
+		if (fcgi_puts(tp, datebuf) == -1)
+			return -1;
 		break;
 	}
-	return NULL;
+	return 0;
 }
