@@ -157,7 +157,7 @@ struct repo_dir {
 	char			*owner;
 	char			*description;
 	char			*url;
-	char			*age;
+	time_t			 age;
 	char			*path;
 };
 
@@ -451,14 +451,9 @@ void sockets_socket_accept(int, short, void *);
 int sockets_privinit(struct gotwebd *, struct socket *);
 
 /* gotweb.c */
-const struct got_error *gotweb_render_content_type(struct request *,
-    const char *);
-const struct got_error
-    *gotweb_render_content_type_file(struct request *, const char *,
-    const char *, const char *);
 void gotweb_get_navs(struct request *, struct gotweb_url *, int *,
     struct gotweb_url *, int *);
-const struct got_error *gotweb_get_time_str(char **, time_t, int);
+int gotweb_render_age(struct template *, time_t, int);
 const struct got_error *gotweb_init_transport(struct transport **);
 const char *gotweb_action_name(int);
 int gotweb_render_url(struct request *, struct gotweb_url *);
@@ -507,8 +502,8 @@ int fcgi_gen_binary_response(struct request *, const uint8_t *, int);
 /* got_operations.c */
 const struct got_error *got_gotweb_flushfile(FILE *, int);
 const struct got_error *got_get_repo_owner(char **, struct request *);
-const struct got_error *got_get_repo_age(char **, struct request *,
-    const char *, int);
+const struct got_error *got_get_repo_age(time_t *, struct request *,
+    const char *);
 const struct got_error *got_get_repo_commits(struct request *, int);
 const struct got_error *got_get_repo_tags(struct request *, int);
 const struct got_error *got_get_repo_heads(struct request *);
