@@ -35,6 +35,7 @@
 
 #else /* Linux, etc... */
 #include <endian.h>
+#include <grp.h>
 #endif
 
 #include <fnmatch.h>
@@ -56,10 +57,31 @@
 #include <sys/file.h>
 #endif
 
+/* POSIX doesn't define WAIT_ANY, so provide it if it's not found. */
+#ifndef WAIT_ANY
+#define WAIT_ANY (-1)
+#endif
+
+/* SOCK_NONBLOCK isn't available across BSDs... */
+#ifndef SOCK_NONBLOCK
+#define SOCK_NONBLOCK 00004000
+#endif
+
+/* On FreeBSD (and possibly others), EAI_NODATA was removed, in favour of
+ * using EAI_NONAME.
+ */
+#ifndef EAI_NODATA
+#define EAI_NODATA EAI_NONAME
+#endif
+
 #ifndef __dead
 #define __dead __attribute__ ((__noreturn__))
 #endif
 
+#ifndef __unused
+#define __unused __attribute__ ((__unused__))
+#endif
+
 #ifndef __OpenBSD__
 #define pledge(s, p) (0)
 #define unveil(s, p) (0)
@@ -69,6 +91,14 @@
 #define cap_enter() (0)
 #endif
 
+#ifndef HAVE_SETRESGID
+#define setresgid(a, b, c) (0)
+#endif
+
+#ifndef HAVE_SETRESUID
+#define setresuid(a, b, c) (0)
+#endif
+
 #ifndef HAVE_LINUX_LANDLOCK_H
 #define landlock_no_fs() (0)
 #else
@@ -116,8 +146,6 @@ void uuid_to_string(uuid_t *, char **, uint32_t *);
 
 #ifdef HAVE_QUEUE_H
 #include <sys/queue.h>
-#else
-#include "compat/queue.h"
 #endif
 
 #ifdef HAVE_TREE_H
@@ -169,6 +197,34 @@ void uuid_to_string(uuid_t *, char **, uint32_t *);
 #define SHA1Final	SHA1_Final
 #endif
 
+/*
+ * The following SA_LEN/SS_LEN dance comes from various source, notably
+ * OpenSMTP by way of OpenNTPD and OpenBGPD (thanks everyone!).  got-portable
+ * has tweaked a lot of the following macros to suit the needs of
+ * got-portable.
+ */
+
+/* From OpenNTPD portable */
+#if !defined(SA_LEN)
+# if defined(HAVE_STRUCT_SOCKADDR_SA_LEN)
+#  define SA_LEN(x)	((x)->sa_len)
+# else
+#  define SA_LEN(x)     ((x)->sa_family == AF_INET6 ? \
+			sizeof(struct sockaddr_in6) : \
+			sizeof(struct sockaddr_in))
+# endif
+
+#endif
+
+/* From OpenBGPD portable */
+#if !defined(SS_LEN)
+# if defined(HAVE_STRUCT_SOCKADDR_STORAGE_SS_LEN)
+#  define SS_LEN(x)  ((x)->ss_len)
+# else
+#  define SS_LEN(x)  SA_LEN((struct sockaddr *)(x))
+# endif
+#endif
+
 #ifndef HAVE_ASPRINTF
 /* asprintf.c */
 int		 asprintf(char **, const char *, ...);
@@ -254,6 +310,11 @@ void		*reallocarray(void *, size_t, size_t);
 void		*recallocarray(void *, size_t, size_t, size_t);
 #endif
 
+#ifndef HAVE_SETPROCTITLE
+/* setproctitle.c */
+void 		 setproctitle(const char *, ...);
+#endif
+
 #ifndef HAVE_FMT_SCALED
 /* fmt_scaled.c */
 int fmt_scaled(long long, char *);
