@@ -102,7 +102,7 @@
 #define GOT_ERR_BRANCH_EXISTS	83
 #define GOT_ERR_MODIFIED	84
 #define GOT_ERR_NOT_REBASING	85
-#define GOT_ERR_EMPTY_REBASE	86
+/* 86 is currently unused */
 #define GOT_ERR_REBASE_COMMITID	87
 #define GOT_ERR_REBASING	88
 #define GOT_ERR_REBASE_PATH	89
