@@ -121,7 +121,7 @@ pack_progress(void *arg, int ncolored, int nfound, int ntrees,
 
 	err = a->progress_cb(a->progress_arg, ncolored, nfound, ntrees,
 	    packfile_size, ncommits, nobj_total, nobj_deltify,
-	    nobj_written, 0, NULL, 0);
+	    nobj_written, 0, NULL, NULL, 0);
 	if (err)
 		return err;
 
@@ -665,13 +665,15 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 	while (!done) {
 		int success = 0;
 		char *refname = NULL;
+		char *errmsg = NULL;
+
 		if (cancel_cb) {
 			err = (*cancel_cb)(cancel_arg);
 			if (err)
 				goto done;
 		}
 		err = got_privsep_recv_send_progress(&done, &bytes_sent,
-		    &success, &refname, &sendibuf);
+		    &success, &refname, &errmsg, &sendibuf);
 		if (err)
 			goto done;
 		if (refname && got_ref_name_is_valid(refname) && success &&
@@ -695,14 +697,16 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 			    ppa.nfound, ppa.ntrees, ppa.packfile_size,
 			    ppa.ncommits, ppa.nobj_total, ppa.nobj_deltify,
 			    ppa.nobj_written, bytes_sent,
-			    refname, success);
+			    refname, errmsg, success);
 			if (err) {
 				free(refname);
+				free(errmsg);
 				goto done;
 			}
 			bytes_sent_cur = bytes_sent;
 		}
 		free(refname);
+		free(errmsg);
 	}
 done:
 	if (sendpid != -1) {
@@ -721,8 +725,8 @@ done:
 		err = got_error_from_errno("close");
 
 	got_ref_list_free(&refs);
-	got_pathlist_free(&have_refs);
-	got_pathlist_free(&their_refs);
+	got_pathlist_free(&have_refs, GOT_PATHLIST_FREE_NONE);
+	got_pathlist_free(&their_refs, GOT_PATHLIST_FREE_NONE);
 	for (i = 0; i < nours; i++)
 		free(our_ids[i]);
 	free(our_ids);
