@@ -179,7 +179,7 @@ send_error(const char *buf, size_t len)
 	size_t i;
 
 	for (i = 0; i < len && i < sizeof(msg) - 1; i++) {
-		if (!isprint(buf[i]))
+		if (!isprint((unsigned char)buf[i]))
 			return got_error_msg(GOT_ERR_BAD_PACKET,
 			    "non-printable error message received from server");
 		msg[i] = buf[i];
@@ -221,10 +221,11 @@ send_ref_status(struct imsgbuf *ibuf, const char *refname, int success,
     struct got_pathlist_head *refs, struct got_pathlist_head *delete_refs)
 {
 	struct ibuf *wbuf;
-	size_t len, reflen = strlen(refname);
+	size_t i, len, reflen, errmsglen = 0;
 	struct got_pathlist_entry *pe;
 	int ref_valid = 0;
-	char *eol;
+	char *eol, *sp;
+	const char *errmsg = "";
 
 	eol = strchr(refname, '\n');
 	if (eol == NULL) {
@@ -233,6 +234,27 @@ send_ref_status(struct imsgbuf *ibuf, const char *refname, int success,
 	}
 	*eol = '\0';
 
+	sp = strchr(refname, ' ');
+	if (sp != NULL) {
+		*sp++ = '\0';
+		errmsg = sp;
+		errmsglen = strlen(errmsg);
+
+		for (i = 0; i < errmsglen; ++i) {
+			if (!isprint((unsigned char)errmsg[i])) {
+				return got_error_msg(GOT_ERR_BAD_PACKET,
+				    "non-printable error message received "
+				    "from the server");
+			}
+		}
+	}
+
+	reflen = strlen(refname);
+	if (!got_ref_name_is_valid(refname)) {
+		return got_error_msg(GOT_ERR_BAD_PACKET,
+		    "unexpected message from server");
+	}
+
 	TAILQ_FOREACH(pe, refs, entry) {
 		if (strcmp(refname, pe->path) == 0) {
 			ref_valid = 1;
@@ -252,7 +274,7 @@ send_ref_status(struct imsgbuf *ibuf, const char *refname, int success,
 		    "unexpected message from server");
 	}
 
-	len = sizeof(struct got_imsg_send_ref_status) + reflen;
+	len = sizeof(struct got_imsg_send_ref_status) + reflen + errmsglen;
 	if (len >= MAX_IMSGSIZE - IMSG_HEADER_SIZE)
 		return got_error(GOT_ERR_NO_SPACE);
 
@@ -266,8 +288,12 @@ send_ref_status(struct imsgbuf *ibuf, const char *refname, int success,
 		return got_error_from_errno("imsg_add SEND_REF_STATUS");
 	if (imsg_add(wbuf, &reflen, sizeof(reflen)) == -1)
 		return got_error_from_errno("imsg_add SEND_REF_STATUS");
+	if (imsg_add(wbuf, &errmsglen, sizeof(errmsglen)) == -1)
+		return got_error_from_errno("imsg_add SEND_REF_STATUS");
 	if (imsg_add(wbuf, refname, reflen) == -1)
 		return got_error_from_errno("imsg_add SEND_REF_STATUS");
+	if (imsg_add(wbuf, errmsg, errmsglen) == -1)
+		return got_error_from_errno("imsg_add SEND_REF_STATUS");
 
 	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
@@ -565,11 +591,7 @@ send_pack(int fd, struct got_pathlist_head *refs,
 
 	err = send_done(ibuf);
 done:
-	TAILQ_FOREACH(pe, &their_refs, entry) {
-		free((void *)pe->path);
-		free(pe->data);
-	}
-	got_pathlist_free(&their_refs);
+	got_pathlist_free(&their_refs, GOT_PATHLIST_FREE_ALL);
 	free(id_str);
 	free(id);
 	free(refname);
@@ -581,12 +603,11 @@ int
 main(int argc, char **argv)
 {
 	const struct got_error *err = NULL;
-	int sendfd;
+	int sendfd = -1;
 	struct imsgbuf ibuf;
 	struct imsg imsg;
 	struct got_pathlist_head refs;
 	struct got_pathlist_head delete_refs;
-	struct got_pathlist_entry *pe;
 	struct got_imsg_send_request send_req;
 	struct got_imsg_send_ref href;
 	size_t datalen, i;
@@ -709,16 +730,8 @@ main(int argc, char **argv)
 
 	err = send_pack(sendfd, &refs, &delete_refs, &ibuf);
 done:
-	TAILQ_FOREACH(pe, &refs, entry) {
-		free((char *)pe->path);
-		free(pe->data);
-	}
-	got_pathlist_free(&refs);
-	TAILQ_FOREACH(pe, &delete_refs, entry) {
-		free((char *)pe->path);
-		free(pe->data);
-	}
-	got_pathlist_free(&delete_refs);
+	got_pathlist_free(&refs, GOT_PATHLIST_FREE_ALL);
+	got_pathlist_free(&delete_refs, GOT_PATHLIST_FREE_ALL);
 	if (sendfd != -1 && close(sendfd) == -1 && err == NULL)
 		err = got_error_from_errno("close");
 	if (err != NULL && err->code != GOT_ERR_CANCELLED)  {
